#include "crasm86.inc"

;TriBase gene codon decoder (v1.2)
;by Cassady Roop
;-------------------------------------------------
;Converts DNA codons to RNA and DNA anticodons and
;displays the amino acid it codes for.
;-------------------------------------------------

dynamic_data	equ	$EAD0		;_asm_exec_ram+5000, to store changing data
arrow_flag	equ	dynamic_data+0	;determines whether in DNA or RNA entry mode
num_bases	equ dynamic_data+1	;keeps track of how many bases entered so far
letters		equ	dynamic_data+2	;holds the characters of base for each DNA,RNA,and aDNA
DNA_codon	equ	dynamic_data+5	;holds displayable codon string, z-terminated
RNA_acodon	equ dynamic_data+9	;...
DNA_acodon	equ dynamic_data+13


.org _asm_exec_ram

	call _runindicoff			;turn off run indicator
	call _flushallmenus			;close open menus
	call _clrlcd				;clear the screen
	xor a
	ld (arrow_flag), a			;set entry field to DNA mode
	jp clear					;clean start
	
	

mainscreen:						;draw user interface
	call _clrlcd				;clear the screen
	ld hl, $0000
	ld (_pencol), hl			;ld screen coords
	ld hl, Title
	call _vputs					;display title
	ld hl, $004B
	ld (_pencol), hl
	ld hl, credittext
	call _vputs					;'by Cassady Roop'
	ld c, $06					;row input for H_line
	call H_line					;make a line at row $0C
	ld hl, $0700
	ld (_pencol), hl
	ld hl, Keys1
	call _vputs					;"EXIT-quit"
	ld hl, $075A
	ld (_pencol), hl
	ld hl, Keys2
	call _vputs					;'MORE-help'
	ld hl, $1305				
	ld (_pencol), hl
	ld hl, DNA_text
	call _vputs					;"DNA codon: "
	ld hl, $1A05				;next text line
	ld (_pencol), hl
	ld hl, aRNA_text
	call _vputs					;"RNA anticodon: "
	ld hl, $2105
	ld (_pencol), hl
	ld hl, aDNA_text
	call _vputs					;"DNA anticodon: "
	ld c, $29					;put line into c
	call H_line					;horizontal line at row c
	call display_codons
	jr codons_displayed
display_codons:					;a function so it is callable
	ld hl, $133F
	ld (_pencol), hl
	ld hl, DNA_codon			
	call _vputs					;disp DNA codon bases
	ld hl, $1A3F
	ld (_pencol), hl
	ld hl, RNA_acodon
	call _vputs					;disp RNA anticodon bases
	ld hl, $213F
	ld (_pencol), hl
	ld hl, DNA_acodon
	call _vputs					;disp DNA anticodon bases
	ret
codons_displayed:
	ld c, $35					;row input for H_line
	call H_line
	inc c
	call H_line					;triple line over fn keys
	inc c
	call H_line
	ld hl, $380B
	ld (_pencol), hl			;coords for menu letters
	ld a, 'C'
	call _vputmap				;'C'
	ld a, $23
	ld (_pencol), a
	ld a, 'G'
	call _vputmap				;'G'
	ld a, $3D
	ld (_pencol), a
	ld a, 'A'
	call _vputmap				;'A'
	ld a, $56
	ld (_pencol), a
	ld a, 'T'
	call _vputmap				;'T'
	ld a, $6E
	ld (_pencol), a
	ld a, 'U'
	call _vputmap				;'U'
	
	ld a, (arrow_flag)			;where should the arrow be?
	cp 1
	jr z, arrow_is_down			;RNA field is active
	ld hl, $1300
	ld (_pencol), hl
	ld a, $05					;ld arrow char
	call _vputmap
	jr keyloop
arrow_is_down:
	ld hl, $1A00
	ld (_pencol), hl
	ld a, $05					;ld arrow char
	call _vputmap				;disp it
	

keyloop:
	call _getkey				;get keypress
	cp kexit
	jp z, Quit
	cp knext
	jp z, Help
	cp kup
	jp z, arrow_up				;go to DNA field
	cp kdown
	jp z, arrow_down			;go to RNA field
	cp kf1
	jp z, place_C				;user placed a C
	cp kf2
	jp z, place_G				;user placed a G
	cp kf3
	jp z, place_A				;user placed a A
	cp kf4
	jp z, place_T				;user placed a T
	cp kf5
	jp z, place_U				;user placed a U
	cp kclear
	jp z, clear					;user pressed clear				
	jr keyloop					;invalid keypress
	
arrow_up:
	xor a
	ld (arrow_flag), a			;set arrow at field 0
	jr clear					;blank the codon fields and reset num_bases to 0, then restart
arrow_down:
	ld a, 1
	ld (arrow_flag), a			;set arrow at field 1
	jr clear					;blank the codon fields

clear:							;clear codons and num_bases, then restart
	call clear_codons			;clear the codon strings
	xor a
	ld (num_bases), a			;clean slate
	jp mainscreen				;redraw
clear_codons:					;sets codon strings to spaces
	ld a, ' '
	ld hl, DNA_codon			;blank the DNA codon
	ld (hl), a
	inc hl
	ld (hl), a
	inc hl
	ld (hl), a
	ld hl, RNA_acodon			;blank the RNA a-codon
	ld (hl), a
	inc hl
	ld (hl), a
	inc hl
	ld (hl), a
	ld hl, DNA_acodon			;blank the DNA a-codon
	ld (hl), a
	inc hl
	ld (hl), a
	inc hl
	ld (hl), a
	xor a
	ld (DNA_codon+3), a			;zero-terminate the strings
	ld (RNA_acodon+3), a
	ld (DNA_acodon+3), a
	ret

place_C:
	ld a, (arrow_flag)
	cp 0						;check if in DNA field
	jr nz, C_rna				;user is in RNA entry field
	ld a, 'C'					;put C in DNA field
	ld (letters), a
	ld a, 'G'					;C in DNA = G in DNA a-codon
	ld (letters+1), a
	ld (letters+2), a			;C in DNA = G in RNA a-codon
	jp place_base
C_rna:							;if user is in RNA field
	ld a, 'G'					;C in RNA = G in DNA
	ld (letters), a
	ld a, 'C'					;C in RNA = C in DNA a-codon
	ld (letters+1), a
	ld (letters+2), a			;put C in RNA field
	jp place_base				;set them


place_G:
	ld a, (arrow_flag)
	cp 0						;is user in DNA field?
	jr nz, G_rna				;user in RNA entry field
	ld a, 'G'					;put G in DNA field
	ld (letters), a
	ld a, 'C'					;G in DNA = C in DNA a-codon
	ld (letters+1), a
	ld (letters+2), a			;G in DNA = C in RNA a-codon
	jp place_base
G_rna:							;user is in RNA field
	ld a, 'C'					;G in RNA = C in DNA
	ld (letters), a
	ld a, 'G'					;G in RNA = G in DNA a-codon
	ld (letters+1), a
	ld (letters+2), a			;put G in RNA
	jp place_base				;set them


place_A:
	ld a, (arrow_flag)
	cp 0						;user in DNA entry field?
	jr nz, A_rna				;user is in RNA field
	ld a, 'A'					;put A in DNA field
	ld (letters), a
	ld a, 'T'					;A in DNA = T in DNA a-codon
	ld (letters+1), a
	ld a, 'U'					;A in DNA = U in RNA a-codon
	ld (letters+2), a
	jp place_base
A_rna:							;if user is in RNA field
	ld a, 'T'					;A in RNA = T in DNA
	ld (letters), a
	ld a, 'A'					;A in RNA = A in DNA a-codon
	ld (letters+1), a
	ld (letters+2), a			;put A in RNA
	jp place_base


place_T:
	ld a, (arrow_flag)
	cp 1						;is user in RNA entry field?
	jp z, mainscreen			;can't place T in RNA
	ld a, 'T'					;put T in DNA
	ld (letters),a
	ld a, 'A'					;T in DNA = A in DNA a-codon
	ld (letters+1),a
	ld (letters+2),a			;T in DNA = A in RNA a-codon
	jp place_base				;set them

place_U:
	ld a, (arrow_flag)
	cp 0						;is user in DNA entry field?
	jp z, mainscreen			;can't place U in DNA
	ld a, 'A'					;U in RNA = A in DNA
	ld (letters),a
	ld a, 'T'					;U in RNA = T in DNA a-codon
	ld (letters+1),a
	ld a, 'U'					;put U in RNA
	ld (letters+2),a
	jp place_base
	


place_base:							
	ld hl, DNA_codon
	ld b, 0
	ld a, (num_bases)			;how many already entered?
	cp b
	jr nz, _2
	inc a
	ld (num_bases), a			;update the count
	ld a, (letters)				;what to put?
	ld (hl), a					;put it in the codon string
	jp _DNAcodon_done			;done setting DNA
_2:
	inc b						
	cp b						;how many already entered? 2?
	jr nz, _3
	inc a
	ld (num_bases), a
	ld a, (letters)
	inc hl
	ld (hl), a
	jp _DNAcodon_done
_3:
	inc a
	ld (num_bases), a
	ld a, (letters)
	inc hl
	inc hl
	ld (hl), a
_DNAcodon_done:
	ld a, (num_bases)			;since it was inc'd earlier in
	dec a						;this routine, dec it to get
								;the original value
	ld hl, DNA_acodon
	ld b, 0
	cp b
	jr nz, _4
	inc a
	ld (num_bases), a
	ld a, (letters+1)			;input base for DNA a-codon
	ld (hl), a
	jp _DNAacodon_done
_4:
	inc b
	cp b
	jr nz, _5
	inc a
	ld (num_bases), a
	ld a, (letters+1)
	inc hl
	ld (hl), a
	jp _DNAacodon_done	
_5:
	inc a
	ld (num_bases), a
	ld a, (letters+1)
	inc hl
	inc hl
	ld (hl), a
_DNAacodon_done:
	ld a, (num_bases)
	dec a
	
	ld hl, RNA_acodon
	ld b, 0
	cp b
	jr nz, _6
	inc a
	ld (num_bases), a
	ld a, (letters+2)			;places RNA a-codon base
	ld (hl), a
	jp mainscreen					;redraw
_6:
	inc b
	cp b
	jr nz, _7
	inc a
	ld (num_bases), a
	ld a, (letters+2)
	inc hl
	ld (hl), a
	jp mainscreen					;redraw	
_7:
	inc a
	ld (num_bases), a
	ld a, (letters+2)
	inc hl
	inc hl
	ld (hl), a
	jr entry_done					;done with codon entry



entry_done:
	call display_codons				;make sure last base displays
	ld hl, DNA_codon				;Lookup routine's input
	call Lookup						;match codon to amino's address
	push hl							;preserve Lookup output
	ld hl, $2E01
	ld (_pencol), hl				;screen coords
	pop hl							;recall amino string address
	call _vputs						;display amino name
	call clear_codons				;reset the codon entry strings
	xor a							;zero a
	ld (num_bases), a				;zero bases entered
	jp keyloop						;goto master keyloop

	
;---------------------------------------------------
;H_LINE - makes horizontal line at row inputted in c
;---------------------------------------------------
H_line:							;horizontal line genrator routine
	xor a						;init counter
	ld hl, $FC00				;video mem start
	ld de, $0010	
H_line_find:	
	cp c						;have we gone far enough?
	jr z, H_line_found
	add hl, de					;next line (hl + 16)
	inc a						;increment line counter
	jr H_line_find				;reloop
H_line_found:					;beginning byte of line found
	xor a						;init counter
H_line_fill:	
	ld (hl), $FF				;8 pixels darkened
	cp 15						;whole line done?
	ret z						;then finish
	inc hl						;next byte
	inc a						;increment counter
	jr H_line_fill				;reloop


Quit:
	call _clrlcd			;clean the screen up first
	ld hl, $0101
	ld (_currow), hl
	ret

Help:
	call _clrlcd
	ld hl, $0000
	ld (_pencol), hl
	ld hl, Title
	call _vputs
	ld c, $08
	call H_line					;horizontal line at row $08
	ld hl, $0A00
	ld (_pencol), hl
	ld hl, Help_Info1
	call _vputs
	ld hl, $1000
	ld (_pencol), hl
	ld hl, Help_Info2
	call _vputs
	ld hl, $1600
	ld (_pencol), hl
	ld hl, Help_Info3
	call _vputs
	ld hl, $1C00
	ld (_pencol), hl
	ld hl, Help_Info4
	call _vputs
	ld hl, $2200
	ld (_pencol), hl
	ld hl, Help_Info5
	call _vputs
	ld hl, $2800
	ld (_pencol), hl
	ld hl, Help_Info6
	call _vputs
	ld hl, $2E00
	ld (_pencol), hl
	ld hl, Help_Info7
	call _vputs
	ld c, $36
	call H_line					;horizontal line at row $36
	ld hl, $3824
	ld (_pencol), hl
	ld hl, Press_Any		;'Press any key'
	call _vputs
	call _getkey			;wait for keypress
	jp mainscreen			;redraw the mainscreen

Title:
	.db "TriBase  1.2",0
credittext:
	.db "by Cassady Roop",0
Keys1:
	.db "EXIT-quit",0
Keys2:
	.db "MORE-help",0
Help_Info1:
	.db "The arrow ",$05," points to the active",0
Help_Info2:
	.db "entry field.  You can enter bases in",0
Help_Info3:
	.db "DNA or RNA and their counterparts",0
Help_Info4:
	.db "will appear in the other fields.  You",0
Help_Info5:
	.db "can't enter T in RNA or U in DNA.",0
Help_Info6:
	.db "When a codon is completed, the amino",0
Help_Info7:
	.db "acid it codes for is displayed.",0
Press_Any:
	.db "Press any key",0
DNA_text:
	.db "DNA codon: ",0
aDNA_text:
	.db "DNA anticodon: ",0
aRNA_text:
	.db "RNA anticodon: ",0
base_C:
	.db "C",0
base_G:
	.db "G",0
base_A:
	.db 'A',0
base_T:
	.db 'T',0
base_U:
	.db 'U',0


; Lookup table routine by Jimmy Mardell
; Input,  HL -> three letter string (uppercase)
; Output, NC, HL -> string
;         CF = error in input string
Lookup:
 	ex de,hl
 	ld bc,$0300 ; B = no of letters, C = acid no
 ; Loop through the three letters to convert the string to a number
 ; between 0-63 (CCC=0, CCG=1, ... TTT=63)
GetAcidNum:
 	push bc
 	ld a,(de) 				; Get next letter in string
 	inc de
 	ld hl,lookup_letters
 	ld bc,4   				; Four valid letters
 	cpir   					; This one is fast...
 	ld h,c 					; H = letter no (0<=h<=3)
 	pop bc
 	scf
	ret nz ; If the letter is none of C, G, A and T, set carry and return
 	ld a,c
 	add a,a
 	add a,a
 	add a,h
 	ld c,a 					; acid=acid*4+letter
 	djnz GetAcidNum
 	ld h,0
 	ld l,a
 	add hl,hl 				; Each ptr is 2 bytes
 	ld de,ptrTable
 	add hl,de
 	call _ldhlind  			; = LD HL,(HL)
 	or a  					; Clear carry - operation succesful
 	ret						;return address of amino in hl

lookup_letters:
	.db "TAGC"  			; Backwards, so C=0, G=1, A=2, T=3
ptrTable:
	.dw CCC,CCC,CCC,CCC,CGC,CGC,CGC,CGC,CAC,CAC,CAC,CAC,CTC,CTG,CTG,CTC
	.dw GCC,GCC,GCC,GCC,GGC,GGC,GGC,GGC,GAC,GAC,GAC,GAC,GTC,GTG,GTG,GTC
	.dw ACC,ACG,ACG,ACT,AGC,AGC,AGC,AGC,GAC,AAG,AAG,GAC,ACT,ATG,ATG,ACT
	.dw GCC,AGC,AGC,GCC,TGC,TGC,TGC,TGC,TAC,TAG,TAG,TAG,TTC,TTG,TTG,TTC
 		;several identical pointer entries because many base
		; combinations code for the same amino acid.  Note:
		; matching uses the DNA codon, not RNA.

CCC:
	.db "Glycine",0			;all CCx codons
CGC:
	.db "Alanine",0			;all CGx codons
CAC:
	.db "Valine",0			;all CAx codons
CTC:
	.db "Glutamic Acid",0	;CTC and CTT
CTG:
	.db "Aspartic Acid",0	;CTG and CTA
GCC:
	.db "Arginine",0		;all GCx codons, TCC, TCT
GGC:
	.db "Proline",0			;all GGx codons
GAC:
	.db "Leucine",0			;all GAx codons, AAC, AAT
GTC:
	.db "Glutamine",0		;GTC and GTT
GTG:
	.db "Histidine",0		;GTG and GTA
ACC:
	.db "Trytophan",0
ACG:
	.db "Cysteine",0		;ACG and ACA
ACT:
	.db "STOP codon",0		;ACT, ATC, ATT
AGC:
	.db "Serine",0			;all AGx codons, TCG, TCA
AAG:
	.db "Phenylalanine",0	;AAG and AAA
ATG:
	.db "Tyrosine",0		;ATG and ATA
TGC:
	.db "Threonine",0		;all TGx codons
TAC:
	.db "Methionine",0
TAG:
	.db "Isoleucine",0		;TAG, TAA, TAT
TTC:
	.db "Lysine",0			;TTC and TTT
TTG:
	.db "Asparagine",0		;TTG and TTA


.end